<?php
/*
 * Name:      Timesheet
 * Directory: timesheet
 * Version:   3.0.1
 * Type:      user
 * UI Name:   Timesheet
 * UI Icon:
 */

/*
 * Copyright (C) 2007-2015, M2X BV
 *
 * Authors: Jean-Paul Saman
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *****************************************************************************/

// MODULE CONFIGURATION DEFINITION
$config = array();
$config['mod_name'] = 'Timesheet';      // name the module
$config['mod_version'] = '3.0.1';       // add a version number
$config['mod_directory'] = 'timesheet'; // tell dotProject where to find this module
$config['mod_setup_class'] = 'CSetupTimesheet';	// the name of the PHP setup class (used below)
$config['mod_type'] = 'user';           // 'core' for modules distributed with dP by standard, 'user' for additional modules from dotmods
$config['mod_ui_name'] = $config['mod_name']; // the name that is shown in the main menu of the User Interface
$config['mod_ui_icon'] = 'communicate.gif'; // name of a related icon
$config['mod_description'] = 'Timesheet module for hour registration'; // some description of the module
$config['mod_config'] = true;            // show 'configure' link in viewmods
$config['mod_main_class'] = 'CTimesheet';// this is the table the system should check for permissions

$config['permission_item_table'] = 'timesheet';
$config['permissions_item_field'] = 'timesheet_id';
$config['permissions_item_label'] = 'timesheet_period';

// show module configuration in 'System Administration|View Modules' 
if (@$a == 'setup') {
    echo w2PshowModuleConfig($config);
}

$config['requirements'] = array(
    array('require' => 'web2project', 'comparator' => '>=', 'version' => '3')
);

class CSetupTimesheet extends w2p_Core_Setup
{
    public function configure() {
        // load module specific configuration page
        global $AppUI;
        $AppUI->redirect( 'm=timesheet&a=configure' );
        return true;
    }

    public function remove() {
        $q = new w2p_Database_Query();
        $q->dropTable('timesheet');
        $q->exec();

        $q->clear();
        $q->dropTable('timesheet_project');
        $q->exec();

	$q->clear();
	$q->dropTable('timesheet_config');
	$q->exec();

        return parent::remove();
    }

    public function upgrade($old_version) {
        // use this to provide upgrade functionality between different versions

 	$result = false;
        switch ($old_version) {
        case 'all': // upgrade from scratch (called from install)
        case '0.9':
        case '1.0':
        case '1.0.1':
        case '1.0.2':
	case '2.0.0':
	case '2.0.1':
	case '2.0.2':
        case '2.0.3':
            $result = $this->_createConfigurationTable();
            if (!$result) {
                db_error();
	        return false;
            }
            $result = $this-> _insertConfigurationData();
            if (!$result) {
                db_error();
	        return false;
            }
	case '3.0.0':
        default:
            // do nothing
        }

        return $result;
    }

    public function install() {
        $result = $this->_checkRequirements();
        if (!$result) {
            return false;
        }

        // Timesheet table
        $result = $this->createTimesheetTable();
        if (!$result) {
            db_error();
	    return false;
        }

        // Timesheet project table
        $result = $this->_createProjectTable();
	if (!$result) {
            db_error();
	    return false;
        }

        // Timesheet configuration table
        $result = $this->_createConfigurationTable();
        if (!$result) {
            db_error();
	    return false;
        }

        $result = $this-> _insertConfigurationData();
        if (!$result) {
            db_error();
	    return false;
        }

	return parent::install();
    }

    private function _createTimesheetTable() {
        $q = new w2p_Database_Query();
        $sql = "( " .
            " `timesheet_id` int(11) unsigned NOT NULL auto_increment," .
            " `timesheet_period` int(11) NOT NULL default '0', " .
            " `timesheet_status` int(4) NOT NULL default '0', " .
            " `timesheet_date` int(11) NOT NULL default '0', " .
            " `timesheet_creator` int(11) NOT NULL default '0', " .
            " `timesheet_worked` int(11) NOT NULL default '0', " .
            " PRIMARY KEY  (`timesheet_id`), " .
            " UNIQUE KEY `timesheet_id` (`timesheet_id`) " .
            ") ENGINE = MYISAM DEFAULT CHARSET=utf8";
        $q->createTable('timesheet');
        $q->createDefinition($sql);
        $result = $q->exec();
        return $result;
    }

    private function _createProjectTable() {
        $q = new w2p_Database_Query();
        $sql = "( " .
            " `timesheet_queue_id` int(11) NOT NULL auto_increment, " .
            " `timesheet_id` int(11) unsigned NOT NULL, " .
            " `timesheet_project` int(11) NOT NULL default '0', " .
            " `timesheet_project_amount` bigint(20) NOT NULL default '0', " .
            " PRIMARY KEY  (`timesheet_queue_id`) " .
            ") ENGINE = MYISAM DEFAULT CHARSET=utf8";
        $q->clear();
        $q->createTable('timesheet_project');
        $q->createDefinition($sql);
        $result = $q->exec();
        return $result;
    }

    private function _createConfigurationTable() {
        $q = new w2p_Database_Query();
	$sql = "( " .
            " `timesheet_config` varchar(25) NOT NULL, " .
            " `timesheet_value`  varchar(25), " .
            " PRIMARY KEY  (`timesheet_config`) " .
            ") ENGINE = MYISAM DEFAULT CHARSET = utf8";
        $q->createTable('timesheet_config');
        $q->createDefinition($sql);
	$result = $q->exec();
        return $result;
    }

    private function _insertConfigurationData() {
	$q = new w2p_Database_Query();
	$q->addTable('timesheet_config');
	$q->addInsert('timesheet_config', 'timesheet_approval_by');
	$q->addInsert('timesheet_value', '0');
	$q->addInsert('timesheet_config', 'timesheet_book_year');
	$q->addInsert('timesheet_value', '2014');
	$q->addInsert('timesheet_config', 'timesheet_sort_order');
	$q->addInsert('timesheet_value', 'Ascending');
	$result = $q->exec();
        return $result;
    }
}

?>
