<?php
/*
 * Copyright (C) 2007-2014, M2X
 *
 * Authors: Jean-Paul Saman
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *****************************************************************************/

/**
 *  @package web2project
 *  @subpackage modules
 *  @version $Revision: 2.0 $
 */

if (!defined('W2P_BASE_DIR')) {
    die('You should not access this file directly.');
}

/**
 * The Timesheet Class
 */
class CTimesheet extends w2p_Core_BaseObject {
    // link variables to the timesheet object (according to the existing columns in the database table timesheet)
    //use null for a new object, so the database automatically assigns an unique id by 'not null'-functionality
    public $timesheet_id = null;
    public $timesheet_period = null;
    public $timesheet_status = 0;
    public $timesheet_date = null;
    public $timesheet_creator = null;
    public $timesheet_worked = null;

    // the constructor of the CTimesheet class, always combined with the table name and the unique key of the table
    public function __construct() {
        parent::__construct('timesheet', 'timesheet_id');
    }

    public function isValid() {
        $baseErrorMsg = get_class($this) . '::store-check failed -';
        if ($this->timesheet_id == 0) {
            $this->_error['timesheet_id'] = $baseErrorMsg . 'timesheet id not set';
        }
        return (count($this->_error)) ? false : true;
    }

    public function loadFull($notUsed = null, $timesheet_id) {
        // loadFull information for this timesheet
    }

    public function load($notUsed = null, $timesheet_id) {
        $q = $this->_getQuery();
        $q->addQuery('*');
        $q->addTable('timesheet');
        $q->addWhere('timesheet_id =' . (int)$timesheet_id);
        $q->loadObject($this, true, false);
    }

    protected function init() {
        $q = $this->_getQuery();
        $q->addTable('timesheet');
        $q->addInsert('timesheet_period,timesheet_status,timesheet_date,timesheet_creator,timesheet_worked',
            $_POST['timesheet_period'].','.$_POST['timesheet_status'].','.
            $_POST['timesheet_date'].','.$_POST['timesheet_creator'].','.
            $_POST['timesheet_worked'], true);
        if (!($q->exec())) {
            $this->_error['init-timesheet'] = db_error();
            return false;
        }
        // TODO: move this to a postCreate hook
        $this->timesheet_id = db_insert_id();
        return true;
    }

    protected function project_purge()
    {
        $q =  $this->_getQuery();
        $q->setDelete('timesheet_project');
        $q->addWhere('timesheet_id = '. (int) $this->timesheet_id);
        if (!($q->exec())) {
            $this->_error['project-purge-timesheet'] = db_error();
            return false;
        }
        return true;
    }

    protected function project_store($workperiod)
    {
        // Get the current timesheet period
        $period = new w2p_Utilities_Date();
	$period->setDate($workperiod,DATE_FORMAT_UNIXTIME);

        $q = $this->_getQuery();
        $q->addQuery('project_id');
        $q->addQuery('project_name');
        $q->addQuery('project_status');
        $q->addTable('projects');
        $q->addOrder('project_name ASC');
        $projects = $q->LoadList();
        foreach ($projects as $row) {
            $t = $this->_getQuery();
            $t->addQuery('task_id');
            $t->addQuery('task_name');
            $t->addQuery('task_owner');
            $t->addQuery('task_start_date');
            $t->addQuery('task_hours_worked');
            $t->addTable('tasks');
            $t->addWhere('task_project = '. $row["project_id"]);
            $t->addOrder('task_start_date DESC');
            $tasks = $t->LoadList();
            $t->Clear();

            $amount = 0;
            foreach ($tasks as $item) {
                // Query the task_log table for actual start dates.
                $t->addQuery('task_log_id');
                $t->addQuery('task_log_name');
                $t->addQuery('task_log_creator');
                $t->addQuery('task_log_date');
                $t->addQuery('task_log_hours');
                $t->addTable('task_log');
                $t->addWhere('task_log_task = '. $item["task_id"]);
                // What if this is different then current user?
                $t->addWhere('task_log_creator = ' . $this->timesheet_creator);
                $t->addOrder('task_log_date DESC');
                // TODO: select on user and date
                $log = $t->LoadList();
                foreach ($log as $logitem) {
                    if (intval($logitem['task_log_date']))
                    {
                        $taskDate = new w2p_Utilities_Date($logitem['task_log_date']);
                        if ($period->GetMonth() == $taskDate->GetMonth()) {
                                $amount = $amount + $logitem["task_log_hours"];
                        }
                        // else do not account the worked hours to the project for this month
                    }
                }
            }
            $sheet = $this->_getQuery();
            $sheet->addTable('timesheet_project');
            $sheet->addInsert('timesheet_id,timesheet_project,timesheet_project_amount',
                $this->timesheet_id.','.$row["project_id"].','.$amount, true);
            if (!($q->exec())) {
                $this->_error['project-purge-timesheet'] = db_error();
                return false;
            }
        }
        return true;
    }

    public function store($unused = null) {
	$this->clearErrors();

        if ($this->timesheet_id != 0 /*&& $this->canEdit()*/) {
            $this->_action='updated'; // FIXME: still needed??

            $q = $this->_getQuery();
            $q->addTable('timesheet');
            $q->addUpdate('timesheet_worked', $this->timesheet_worked);
            $q->addWhere('timesheet_id = ' . $this->timesheet_id);
            if (!($q->exec())) {
                $this->_error['update-timesheet'] = db_error();
                return false;
            }
            return true;
        }
        else if ($this->canCreate()) {
            $q = $this->_getQuery();
            $q->addTable('timesheet');
            $q->addInsert('timesheet_period,timesheet_status,timesheet_date,timesheet_creator,timesheet_worked',
                $this->timesheet_period.','.$this->timesheet_status.','.
                $this->timesheet_date.','.$this->timesheet_creator.',' .
                $this->timesheet_worked, true);
            if (!($q->exec())) {
                $this->_error['store-timesheet'] = db_error();
                return false;
            }
            return true;
        }
        return false;
    }

    // overload the delete method of the parent class for adaptation for timesheet's needs
    public function delete($unused = null) {
        $this->clearErrors();
        // FIXME: this check does not work
        //if (!$this->canDelete())
        //   return false;

        $q = $this->_getQuery();
        $q->setDelete('timesheet_project');
        $q->addWhere('timesheet_id = '. $this->timesheet_id );
        if (!($q->exec())) {
            $this->_error['delete-timesheet-project'] = db_error();
            return false;
        }
        $q->clear();
        $q->setDelete('timesheet');
        $q->addWhere('timesheet_id = '. $this->timesheet_id );
        if (!($q->exec())) {
            $this->_error['delete-timesheet'] = db_error();
            return false;
        }
        return true;
    }

    public function change_status($status) {
        $this->clearErrors();
        //if (!$this->canEdit())
        //    return false;

        //if ($timesheet_id == 0) {
	//    if (!$this->canCreate())
        //        return false;
        //    $this->init();
	//    $this->load(null, $timesheet_id);
        //}

        $q = $this->_getQuery();
        $q->addTable('timesheet');
        $q->addUpdate('timesheet_status', $status);
        $q->addWhere('timesheet_id = ' . $this->timesheet_id);
        if (!($q->exec())) {
            $this->_error['change-status-timesheet'] = db_error();
            return false;
        }
        return true;
    }

    public function change_period($period) {
        $this->clearErrors();
        //if (!$this->canEdit())
        //    return false;
        if ($timesheet_id == 0) {
	    if (!$this->canCreate())
                return false;
            $this->init();
        }

        $q = $this->_getQuery();
        $q->addTable('timesheet');
        $q->addUpdate('timesheet_period', $period);
        $q->addWhere('timesheet_id = ' . $this->timesheet_id);
        if (!($q->exec())) {
             $this->_error['change-period-timesheet'] = db_error();
             return false;
        }
        $purge_result = $this->project_purge();
        $store_result = $this->project_store($period);
        $timesheet_period = $period;
        return $purge_result && $store_result;
    }

    public function get_config() {
        // Read values from table
        $config = null;
        $q = $this->_getQuery();
        $q->addQuery('*');
        $q->addTable('timesheet_config');
        if (!$q->exec()) {
            $AppUI->setMsg(db_error(), UI_MSG_ERROR);
            return $config;
        }
        while ($row = $q->fetchRow()) {
            $config[$row['timesheet_config']] = $row['timesheet_value'];
        }
        $q->clear();
        return $config;
    }
}
?>
